# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self


class FileInfo(BaseModel):
    """
    FileInfo
    """

  # noqa: E501
    name: StrictStr
    size: Optional[StrictInt] = None
    compressed_size: Optional[StrictInt] = None
    range_start: Optional[StrictInt] = None
    creation_time: Optional[datetime] = None
    compress_type: Optional[StrictInt] = None
    __properties: ClassVar[List[str]] = [
        "name", "size", "compressed_size", "range_start", "creation_time",
        "compress_type"
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FileInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict['size'] = None

        # set to None if compressed_size (nullable) is None
        # and model_fields_set contains the field
        if self.compressed_size is None and "compressed_size" in self.model_fields_set:
            _dict['compressed_size'] = None

        # set to None if range_start (nullable) is None
        # and model_fields_set contains the field
        if self.range_start is None and "range_start" in self.model_fields_set:
            _dict['range_start'] = None

        # set to None if creation_time (nullable) is None
        # and model_fields_set contains the field
        if self.creation_time is None and "creation_time" in self.model_fields_set:
            _dict['creation_time'] = None

        # set to None if compress_type (nullable) is None
        # and model_fields_set contains the field
        if self.compress_type is None and "compress_type" in self.model_fields_set:
            _dict['compress_type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FileInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "size": obj.get("size"),
            "compressed_size": obj.get("compressed_size"),
            "range_start": obj.get("range_start"),
            "creation_time": obj.get("creation_time"),
            "compress_type": obj.get("compress_type")
        })
        return _obj



"""Delft3D module of the API."""
from typing import Optional, Union

from inductiva import simulators, types


class Delft3D(simulators.Simulator):
    """Class to invoke a generic Delft3D simulation on the API."""

    _default_delft3d_device = "cpu"

    def __init__(
        self,
        /,
        version: Optional[str] = None,
        use_dev: bool = False,
    ):
        super().__init__(version, use_dev, self._default_delft3d_device)
        self.simulator = "arbitrary_commands"
        self.simulator_name_alias = "delft3d"

    def run(self,
            input_dir: Optional[str],
            *,
            commands: Optional[types.Commands] = None,
            shell_script: Optional[str] = None,
            on: types.ComputationalResources,
            storage_dir: Optional[str] = "",
            resubmit_on_preemption: bool = False,
            remote_assets: Optional[Union[str, list[str]]] = None,
            project: Optional[str] = None,
            time_to_live: Optional[str] = None,
            on_finish_cleanup: Optional[Union[str, list[str]]] = None,
            **kwargs):
        """Run the simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            on: The computational resource to launch the simulation on.
            storage_dir: Directory for storing results.
            remote_assets: Additional remote files that will be copied to
                the simulation directory.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiated with
                `spot=True`.
            project: Name of the project to which the task will be
                assigned. If None, the task will be assigned to
                the default project. If the project does not exist, it will be
                created.
            time_to_live: Maximum allowed runtime for the task, specified as a
                string duration. Supports common time duration formats such as
                "10m", "2 hours", "1h30m", or "90s". The task will be
                automatically terminated if it exceeds this duration after
                starting.
            commands: List of commands to run the simulation. Cannot be used
                with `shell_script`.
            shell_script: Name of the shell script to run the simulation.
                Cannot be used with `commands`.
            on_finish_cleanup :
                Optional cleanup script or list of shell commands to remove
                temporary or unwanted files generated during the simulation.
                This helps reduce storage usage by discarding unnecessary
                output.
                - If a string is provided, it is treated as the path to a shell
                script that must be included with the simulation files.
                - If a list of strings is provided, each item is treated as an
                individual shell command and will be executed sequentially.
                All cleanup actions are executed in the simulation's working
                directory, after the simulation finishes.
                Examples:
                    on_finish_cleanup = "my_cleanup.sh"

                    on_finish_cleanup = [
                        "rm -rf temp_dir",
                        "rm -f logs/debug.log"
                    ]
            other arguments: See the documentation of the base class.
        """
        if (commands is None) == (shell_script is None):
            raise ValueError("You must provide exactly one of 'commands' or "
                             "'shell_script'.")

        if shell_script is not None:
            #Check if the shell script exists

            self._input_files_exist(input_dir=input_dir,
                                    remote_assets=remote_assets,
                                    shell_script=shell_script)
            commands = [f"bash {shell_script}"]

        return super().run(input_dir,
                           on=on,
                           commands=commands,
                           storage_dir=storage_dir,
                           resubmit_on_preemption=resubmit_on_preemption,
                           remote_assets=remote_assets,
                           project=project,
                           time_to_live=time_to_live,
                           on_finish_cleanup=on_finish_cleanup,
                           **kwargs)



"""List the storage contents via CLI."""

import argparse
import textwrap

from inductiva import storage


def listdir(args):
    """List the user's remote storage contents."""
    max_results = None if args.all else args.max_results
    storage.listdir(args.path, max_results, args.order_by, args.sort_order)


def register(parser):
    """Register the list user's remote storage command."""

    subparser = parser.add_parser("list",
                                  aliases=["ls"],
                                  help="List remote storage contents.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = (
        "The `inductiva storage list` command provides an overview"
        " of your data on Inductiva remote storage.\n"
        "It lists all files and folders in a specified path, allowing you to "
        "control the maximum number of results,\n"
        "the ordering criteria, and the sorting order.\n")
    subparser.add_argument("path", default="/", type=str, nargs="?")
    subparser.add_argument("-m", "--max-results", default=10, type=int)
    subparser.add_argument("-o",
                           "--order-by",
                           default="creation_time",
                           type=str,
                           choices=["creation_time", "size"],
                           help="Order by creation_time or size.")
    subparser.add_argument("-s",
                           "--sort-order",
                           default="desc",
                           type=str,
                           choices=["desc", "asc"],
                           help="Sorting order (desc or asc).")
    subparser.add_argument("--all",
                           action="store_true",
                           help="List all results, ignoring --max-results.")

    subparser.epilog = textwrap.dedent("""\
        examples:
            # List the 10 largest folders sorted by size
            $ inductiva storage list --max-results 10 --order-by size --sort-order desc

            NAME                             SIZE           CREATION TIME
            0bet8jrpp2gz974n42nsd9n2p/       56.11 MB       06 Feb, 11:32:29
            05ujj5m0ytdkckxwk1tq1b5io/       27.93 MB       08 Feb, 09:19:44
            6a2h1wnxywpea8jfoxiikdjf7/       26.49 MB       07 Feb, 13:47:03
            f8joznwc9xf9a4nypcaei6v2s/       12.79 MB       07 Feb, 09:16:55
            dpq2cv6b5f9p1c77nc8anjo10/       12.00 MB       08 Feb, 09:39:31
            r4kerxf4b53krgn0s3fyece3b/       11.92 MB       07 Feb, 11:47:48
            j9qzrpiohgt7x97od3tw4wccd/       11.74 MB       07 Feb, 11:47:46
            iqi71gonoacfj7fknox3rvnq2/       11.52 MB       07 Feb, 11:47:45
            dxmnxdrfrv84pfbzbvm9v0dat/       11.43 MB       07 Feb, 11:47:43
            bgtwgnnyq5qa5hecegzdx6okr/       11.36 MB       07 Feb, 11:47:40
    """)

    subparser.set_defaults(func=listdir)
